using System;
using System.IO;
using System.Collections.Generic;
using System.Linq;
using System.Text.RegularExpressions;
using System.Data.SQLite;
//#Include:System.Data.dll
//#Include:System.Data.SQLite.dll

namespace PandoraBox
{
    public class CBARTICLEPlugin : IPluginMacro
    {
        Regex _block = new Regex(@"\[(GOOGLE|BING|YANDEX)(FREE){0,1}ARTICLE(?<mp>#\d+)*\]", RegexOptions.Compiled);
        Regex _customBlock = new Regex(@"\[MYCB\(([^)]+)\)(FREE){0,1}ARTICLE(?<mp>#\d+)*\]", RegexOptions.Compiled);

        //сколько минимум предложений должно быть в статье / how many sentances must be in article minimum
        int _minSentances = 20;
        //сколько максимум предложений должно быть в статье / how many sentances must be in article maximum
        int _maxSentances = 99;

        public string Execute(string template, PluginMacroArgs args)
        {
            template = Load(template, _block, args);
            template = Load(template, _customBlock, args);
            return template;
        }

        private string Load(string template, Regex regex, PluginMacroArgs args)
        {
            return regex.Replace(template, delegate (Match match)
            {
                var type = match.Groups[1].Value + ".Directory";
                String typePath = GetIniParameter(type, args);
                bool dbMode = GetIniParameter("Main.UseDatabase", args) == "1";
                bool isFreeText = match.Groups[2].Value.Length > 0;
                String key = CleanKey(GetKey(args, match));
                //получаем список использованных элементов
                HashSet<string> usedSentances = args.Context[typePath + "UniqSentList"] as HashSet<string>;
                if (usedSentances == null)
                    args.Context[typePath + "UniqSentList"] = usedSentances = new HashSet<String>();
                //получаем сохраненный ключ контекста
                string oldKey = args.Context[typePath + "UniqSentKey"] as string;
                if (oldKey == null)
                    args.Context[typePath + "UniqSentKey"] = oldKey = "";
                //если ключ поменялся, 
                if (key != oldKey)
                {
                    usedSentances.Clear();
                    args.Context[typePath + "UniqSentKey"] = key;
                }
                String article;
                if (!dbMode)
                {
                    String file = Path.Combine(GetIniParameter("Main.DataFolder", args), typePath, GetSub(key.Trim()), key);
                    if (!File.Exists(file)) //текста по ключу нет, выводим стандартным макросом текста
                        return isFreeText ? "[FREETEXT-3-5/p]" : "[TEXT-3-5/p]";
                    article = GetFileArticle(usedSentances, file);
                }
                else
                {
                    article = GetDBArticle(usedSentances, args, GetIniParameter("Main.DataFolder", args), typePath.Replace(".", "_"), key);
                }
                if (article == null) //статей по требованиям нет, выводим стандартным макросом текста
                    return isFreeText ? "[FREETEXT-3-5/p]" : "[TEXT-3-5/p]";
                if (!isFreeText) article = "{INSERTKEYS}" + article + "{/INSERTKEYS}";
                article = "{PARAGRAPH}" + article + "{/PARAGRAPH}"; //добавляем параграфы
                return article;
            });
        }

        private string GetIniParameter(String name, PluginMacroArgs args)
        {
            String root = args.Context[name] as String;
            if (root == null)
            {
                root = File.ReadAllLines("ContentBox.ini")
                    .FirstOrDefault(x => x.StartsWith(name + "=", StringComparison.InvariantCultureIgnoreCase));
                root = root != null ? root.Split('=')[1] : "";
                args.Context[name] = root;
            }
            return root;
        }

        private string GetSub(string key)
        {
            return key.Length <= 2 ? key : key.Substring(0, 2).Trim();
        }

        private string GetFileArticle(HashSet<string> usedSentances, String file)
        {
            var lines = Shuffle(File.ReadAllLines(file).ToList());
            for (int i = 0; i < lines.Count; i++)
            {
                var sentances = lines[i].Split('|');
                if (sentances.Length >= _minSentances && !usedSentances.Contains(lines[i]))
                {
                    usedSentances.Add(lines[i]);
                    return String.Join(" ", sentances.Take(_maxSentances));
                }
            }
            return null;
        }

        private string GetKeysFile(PluginMacroArgs args)
        {
            return args.Context["$dbname"] as String ?? "content.db";
        }

        private string GetDBArticle(HashSet<string> usedSentances, PluginMacroArgs args, String dbDir, String table, String key)
        {
            //connect
            var contextKey = "SQLiteConnect";
            SQLiteConnection connection = args.Context[contextKey] as SQLiteConnection;
            if (connection == null)
            {
                connection = new SQLiteConnection("Data Source = \"" + dbDir + "/" + GetKeysFile(args) + "\"; Journal Mode=Off;");
                connection.Open();
                args.Context[contextKey] = connection;
            }

            //reader command
            contextKey = "SQLiteCommand" + table;
            SQLiteCommand reader = args.Context[contextKey] as SQLiteCommand;
            if (reader == null)
            {
                reader = new SQLiteCommand("select content from " + table + " where key = :key", connection);
                reader.Parameters.Add(new SQLiteParameter("key", ""));
                args.Context[contextKey] = reader;
                if (!TableExists(table, connection)) reader.CommandText = null;
            }
            if (reader.CommandText == null) return ""; //no table

            reader.Parameters[0].Value = key;
            var content = (reader.ExecuteScalar() as string) ?? "";
            var lines = Shuffle(content.Split(new[] { '\n' }, StringSplitOptions.RemoveEmptyEntries).ToList());
            for (int i = 0; i < lines.Count; i++)
            {
                var sentances = lines[i].Split('|');
                if (sentances.Length >= _minSentances && !usedSentances.Contains(lines[i]))
                {
                    usedSentances.Add(lines[i]);
                    return String.Join(" ", sentances.Take(_maxSentances));
                }
            }
            return null;
        }

        private bool TableExists(String table, SQLiteConnection connection)
        {
            return new SQLiteCommand("SELECT count(*) FROM sqlite_master WHERE name='" + table + "'", connection).ExecuteScalar().ToString() == "1";
        }

        private string GetKey(PluginMacroArgs args, Match match)
        {
            int keyIndex;
            string pos = null;
            var result = args.Key;
            if (args.KeyParts != null && (pos = match.Groups["mp"].Value) != "" && (keyIndex = int.Parse(pos.Trim('#'))) > 0)
                result = args.KeyParts[keyIndex > args.KeyParts.Length ? 0 : keyIndex - 1];
            return result;
        }

        private string CleanKey(String key)
        {
            String key1 = key;
            key1 = String.Join("", key1.Split(Path.GetInvalidPathChars()));
            key1 = String.Join("", key1.Split(Path.GetInvalidFileNameChars()));
            return key1;
        }

        private static List<T> Shuffle<T>(List<T> items)
        {
            if (items.Count < 2) return items;

            for (int i = 0; i < items.Count; i++)
            {
                int j = Rnd.InnerObject.Next(items.Count);
                T temp = items[i];
                items[i] = items[j];
                items[j] = temp;
            }
            return items;
        }

        public ushort Level { get { return 5; } }
        public String Name { get { return "CBARTICLEPlugin"; } }
    }
}